# Author:  Chenghua Shao
# Date:    2025-11-10
# Updates:

"""
Wrapper to parse FindGeo output files
"""

import json
import os
import sys
import logging
from collections import OrderedDict
from mmcif.io.IoAdapterCore import IoAdapterCore
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wwpdb.utils.dp.metal.metal_util.readRef import readRefCoordNum, readRefCoordMap, readRefRedOx
else:
    sys.path.append(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "metal_util"))
    from readRef import readRefCoordNum, readRefCoordMap, readRefRedOx  # noqa: E402

logger = logging.getLogger(__name__)


class ParseFindGeo:
    """Wrapper to parse FindGeo output files
    Example usage:
    pFG = ParseFindGeo(/path/to/findgeo/output/folder, input_format="cif")
    pFG.parse()
    pFG.report("findgeo_report.json")
    """
    def __init__(self, folder, input_format="cif"):
        self.folder = folder
        self.input_format = input_format
        self.l_sites = []
        self.d_coord_num = readRefCoordNum()
        self.d_coord_map = readRefCoordMap("FindGeo")
        (self.d_redox, self.d_oxi) = readRefRedOx()

    def parse(self):
        """
        parse FindGeo output folder to extract top hit coordination geometry for each site.
        1. iterate through each site folder in self.folder
        2. for each site folder, parse findgeo.out to get top hit coordination geometry
        3. parse findgeo.input to get metal atom information
        4. store the results in self.l_sites
        5. sort self.l_sites by metal, chain, residue, sequence, icode
        """
        l_folder = os.listdir(self.folder)
        for name in l_folder:
            if name != "data" and "_" in name:
                d_tophit = self.parseOneSite(name)
                if not d_tophit:
                    continue
                d_tophit = self.amend(d_tophit)
                logger.info("add row %s", d_tophit)
                self.l_sites.append(d_tophit)

        if self.l_sites:
            self.sort()
        else:
            logger.warning("no metal sites parsed in %s", self.folder)

    def amend(self, d_tophit):
        """
        amend d_tophit with additional information from reference data
        1. add generic geometry name from the coordination class mapping reference
        2. check whether the coordination number is allowd in the coordination number reference

        :param d_tophit: dict with top hit information
        :return: amended dict with additional information
        """
        geom = d_tophit["class"]
        if geom in self.d_coord_map:
            pdb_geom = self.d_coord_map[geom]["pdb_geom"]
            d_tophit["class_generic"] = pdb_geom
        else:
            d_tophit["class_generic"] = ""

        metal = d_tophit["metalElement"]
        if metal in self.d_coord_num:
            allowed_coord_num = self.d_coord_num.get(metal)
            if d_tophit["coordination"] in allowed_coord_num:
                d_tophit["coordination_number_allowed"] = "YES"
            else:
                d_tophit["coordination_number_allowed"] = "NO"
        else:
            d_tophit["coordination_number_allowed"] = ""

        if metal in self.d_redox:
            d_tophit["redox_active"] = self.d_redox.get(metal)
        else:
            d_tophit["redox_active"] = ""

        if metal in self.d_oxi:
            d_tophit["oxidation_state"] = self.d_oxi.get(metal)
        else:
            d_tophit["oxidation_state"] = ""

        return d_tophit

    def parseOneSite(self, site_name):
        """
        parse one site folder to extract top hit coordination geometry and metal atom information

        :param site_name: site folder name, e.g. Cm_201__8907_DA/
        :return: dict with top hit information, None if parsing fails
        """
        subfolder = os.path.join(self.folder, site_name)
        if not os.path.isdir(subfolder):
            return None

        logger.info("to process subfolder %s", subfolder)
        l_subfolder = os.listdir(subfolder)
        if ("findgeo.out" not in l_subfolder):
            logger.warning("failed to find findgeo.out in %s", subfolder)
            return None

        d_tophit = self.parseFindGeoOutPut(os.path.join(subfolder, "findgeo.out"))
        if not d_tophit:
            return None

        t_atom = ()
        filepath_input = os.path.join(subfolder, "findgeo.input")
        if not os.path.isfile(filepath_input):
            logger.error("failed to find %s in %s", "findgeo.input", subfolder)
            return None
        if self.input_format == "pdb":
            t_atom = self.parseFindGeoPdbInput(filepath_input)
        elif self.input_format == "cif":
            t_atom = self.parseFindGeoCifInput(filepath_input)
        else:
            logger.error("unsupported input format: %s", self.input_format)
            t_atom = ()

        if not t_atom or len(t_atom) != 6:
            return None
        else:
            (ccd_id, atom_label, chain, res_num, ins, alt) = t_atom  # pylint: disable=unbalanced-tuple-unpacking

        element = site_name.split("_")[0]
        d_tophit["metal"] = atom_label
        d_tophit["metalElement"] = element.capitalize()
        d_tophit["chain"] = chain
        d_tophit["residue"] = ccd_id
        d_tophit["sequence"] = res_num
        d_tophit["icode"] = ins
        d_tophit["altloc"] = alt
        return d_tophit

    def parseFindGeoOutPut(self, filepath):
        """
        parse findgeo.out file to extract top hit coordination geometry
        1. coordination number
        2. best geometry class, class abbreviation, tag, rmsd

        :param filepath: path to findgeo.out file
        :return: dict with top hit information, empty dict if parsing fails
        """
        b_found_coord = False
        best_geo_name = None
        if not os.path.isfile(filepath):
            logger.error("failed to access %s", filepath)
            return {}
        logger.info("to process %s", filepath)
        d_tophit = {}
        with open(filepath) as file:
            l_hit = []
            for line in file:
                if line.startswith("Coordination number"):
                    l_line = line.strip().split(':')
                    d_tophit["coordination"] = l_line[-1].strip()
                    logger.info("found coordination number %s", d_tophit["coordination"])
                    b_found_coord = True
                if "-" in line and "|" in line:
                    l_line = line.strip().split("|")
                    if len(l_line) == 3:
                        d_hit = {}
                        d_hit["class_abbr"] = l_line[0].split("-")[0].strip().upper()
                        d_hit["class"] = l_line[0].split("-")[1].strip().lower()
                        d_hit["tag"] = l_line[1].strip()
                        d_hit["rmsd"] = l_line[2].strip()
                        logger.info("found coordination geometry %s, %s with tag %s RMSD %s",
                                    d_hit["class_abbr"], d_hit["class"], d_hit["tag"], d_hit["rmsd"])
                        l_hit.append(d_hit)
                if line.startswith("Best geometry"):
                    _tmp = line.strip().split(":")[1]
                    if "(Regular)" in _tmp:
                        best_geo_name = _tmp.split("(Regular)")[0].strip().lower()
                    elif "(Distorted)" in _tmp:
                        best_geo_name = _tmp.split("(Distorted)")[0].strip().lower()
                    elif "(Irregular)" in _tmp:
                        # best_geo_name = _tmp.split("(Irregular)")[0].strip().lower()
                        logger.info("best geometry is irregular in %s, use 'irregular' and ignore geometry parameters", filepath)
                        best_geo_name = "irregular"
                    else:
                        logger.warning("cannot find best geometry format in %s, use 'undetected' by default", filepath)
                        best_geo_name = "undetected"
                    logger.info("best geoemtry is %s", best_geo_name)

            if not b_found_coord:
                logger.warning("could not find coordination number in %s", filepath)
                return None

            if not l_hit:
                logger.warning("could not find any geometry hits in %s", filepath)
                return None

            if not best_geo_name:
                logger.warning("could not find best geometry in %s", filepath)
                return None

            if best_geo_name == "undetected":
                d_tophit["class"] = "undetected"
                d_tophit["class_abbr"] = ""
                d_tophit["tag"] = "None"
                d_tophit["rmsd"] = ""
                logger.warning("best geometry is undetected in %s, no geometry parameters output", filepath)
                return d_tophit

            if best_geo_name == "irregular":
                d_tophit["class"] = "irregular"
                d_tophit["class_abbr"] = ""
                d_tophit["tag"] = "Irregular"
                d_tophit["rmsd"] = ""
                logger.warning("best geometry is irregular in %s, no geometry parameters output", filepath)
                return d_tophit

            for d_hit in l_hit:
                if d_hit["class"] == best_geo_name:
                    for item in ["class_abbr", "class", "tag", "rmsd"]:
                        d_tophit[item] = d_hit[item]
                    logger.info("found best geometry %s in %s", best_geo_name, filepath)
                    return d_tophit

            logger.warning("could not find best geometry %s in hits in %s", best_geo_name, filepath)
            return None

    def parseFindGeoPdbInput(self, filepath):
        """
        parse findgeo.input file in pdb format to extract metal atom information

        :param filepath: path to findgeo.input file, if the input was in pdb format
        :return: tuple (ccd_id, atom_label, chain, res_num, ins, alt), empty tuple if parsing fails
        """
        logger.info("to process %s", filepath)
        with open(filepath) as file:
            line = file.readline()
            if line.startswith("ATOM") or line.startswith("HETATM"):
                if len(line.strip()) >= 54:
                    atom_label = line[12:16].strip()
                    alt = line[16:17]
                    ccd_id = line[17:20].strip()
                    chain = line[21:22]
                    res_num = line[22:26].strip()
                    ins = line[26:27]
                    logger.info("found ccd_id %s", ccd_id)
                    return (ccd_id, atom_label, chain, res_num, ins, alt)
        logger.error("failed to process %s", filepath)
        return ()

    def parseFindGeoCifInput(self, filepath):
        """
        parse findgeo.input file in cif format to extract metal atom information

        :param filepath: path to findgeo.input file, if the input was in cif format
        :return: tuple (ccd_id, atom_label, chain, res_num, ins, alt), empty tuple if parsing fails
        """
        logger.info("to run mmCIF parser on findgeo.input %s", filepath)
        d_metal_row = self.parseMmcif(filepath)
        if d_metal_row:
            atom_label = d_metal_row.get("label_atom_id", "").strip()
            alt = d_metal_row.get("label_alt_id", "").strip()
            ccd_id = d_metal_row.get("label_comp_id", "").strip()
            ins = d_metal_row.get("pdbx_PDB_ins_code", "").strip()
            res_num = d_metal_row.get("auth_seq_id", "").strip()
            chain = d_metal_row.get("auth_asym_id", "").strip()
            logger.info("found ccd_id %s", ccd_id)
            return (ccd_id, atom_label, chain, res_num, ins, alt)
        else:
            logger.info("to parse findgeo.input by column guess on: %s", filepath)
            with open(filepath) as file:
                for line in file:
                    if line.startswith("ATOM") or line.startswith("HETATM"):
                        l_line = line.strip().split()
                        atom_label = l_line[3]
                        alt = l_line[4]
                        ccd_id = l_line[5]
                        ins = l_line[9]
                        res_num = l_line[15]
                        chain = l_line[17]
                        logger.info("found ccd_id %s", ccd_id)
                        return (ccd_id, atom_label, chain, res_num, ins, alt)
        logger.error("failed to process %s", filepath)
        return ()

    def parseMmcif(self, fp):
        """
        parse mmcif file to extract atom site information

        :param fp: file path to mmcif file
        :return: dict with 1st row (metal atom) atom site information
        """
        io = IoAdapterCore()
        l_dc = io.readFile(fp)
        if not l_dc:
            logger.error("failed to read mmcif file: %s", fp)
            return {}
        dc0 = l_dc[0]

        if 'atom_site' not in dc0.getObjNameList():
            logger.error("no atom_site category found in mmcif file: %s", fp)
            return {}
        c_atom_site = dc0.getObj('atom_site')
        d_metal_row = c_atom_site.getRowAttributeDict(0)

        if "auth_asym_id" not in d_metal_row:
            logger.error("failed to find auth_asym_id in atom_site category in mmcif file: %s", fp)
            return {}

        return d_metal_row

    def sort(self):
        """
        sort self.l_sites by metal, chain, residue, sequence, icode, altloc,
        coordination, class, class_abbr, tag, rmsd
        """
        key_order = ["metal", "metalElement", "chain", "residue", "sequence", "icode", "altloc",
                     "coordination", "class", "class_abbr", "class_generic", "tag", "rmsd",
                     "coordination_number_allowed", "redox_active", "oxidation_state"]
        l_sorted = []
        for d_row in self.l_sites:
            d_row_sorted = OrderedDict((key, d_row[key]) for key in key_order if key in d_row)
            l_sorted.append(d_row_sorted)
        self.l_sites = l_sorted

    def report(self, filepath_json):
        """
        write self.l_sites to a json file

        :param filepath_json: path to output json file
        """
        logger.info("to write report to %s", filepath_json)
        with open(filepath_json, "w") as file:
            json.dump(self.l_sites, file, indent=4)



# Author:  Chenghua Shao
# Date:    2025-11-10
# Updates:

"""
Wrapper to run Acedrg with arguments
"""

import logging
import os
import sys
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wwpdb.utils.dp.metal.metal_util.run_command import MetalCommandExecutionError, run_command  # noqa: E402
else:
    sys.path.append(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "metal_util"))
    from run_command import MetalCommandExecutionError, run_command  # noqa: E402

logger = logging.getLogger(__name__)


class RunAcedrg:
    """Wrapper to run Acedrg with arguments
    """
    def __init__(self, d_args):
        self.d_args = d_args
        if not self.d_args["acedrg_exe"]:
            logger.info("%s is called without explicit Acedrg executable, to find in CCP4", self.__class__.__name__)
            ccp4_dir = os.getenv("CCP4", default=None)
            if not ccp4_dir:
                raise KeyError("Environment variable 'CCP4' not found")
            acedrg_exe = os.path.join(ccp4_dir, "bin", "acedrg")
            if not os.path.exists(acedrg_exe):
                raise FileNotFoundError("Acedrg executable not found in CCP4 bin/ folder")
            self.d_args["acedrg_exe"] = acedrg_exe
            logger.info("use Acedrg executable at %s", acedrg_exe)

    def run(self):
        """
        run Acedrg with arguments in d_args
        example command:
            Acedrg
            --mmcif 1PT.cif
            --out 1PT_acedrg  # name root of the output files
        :return: stdout from MetalCoord if successful, otherwise None
        :rtype: str or None
        """
        l_command = [self.d_args["acedrg_exe"]]
        l_command.extend(["--mmcif", self.d_args["mmcif"]])
        l_command.extend(["--out", self.d_args["out"]])

        logger.info("to run Acedrg full command:\n %s", ' '.join(l_command))
        try:
            cmd_stdout = run_command(l_command)
            logger.info("finished running Acedrg on %s", self.d_args["mmcif"])
            return cmd_stdout
        except MetalCommandExecutionError as e:
            logger.error(f"MetalCommandExecutionError: {e}")
            return None


def main():
    fp_in = "0KA.cif"
    fp_out_root = "acedrg/acedrg"

    d_args = {"acedrg_exe": None,
              "mmcif": fp_in,
              "out": fp_out_root,
              }

    try:
        rAG = RunAcedrg(d_args)
    except Exception as e:
        print(e)
        sys.exit(1)
    cmd_stdout = rAG.run()
    print(cmd_stdout)


if __name__ == "__main__":
    main()



# Author:  Chenghua Shao
# Date:    2025-11-10
# Updates:

"""
Wrapper to run FindGeo with arguments similar to command line
"""

import logging
import os
import sys
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wwpdb.utils.dp.metal.metal_util.run_command import run_command, MetalCommandExecutionError  # noqa: E402
else:
    sys.path.append(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "metal_util"))
    from run_command import run_command, MetalCommandExecutionError  # noqa: E402

logger = logging.getLogger(__name__)


class RunFindGeo:
    """Wrapper to run FindGeo with arguments similar to command line
    Example usage:
    d_args = {
        "excluded-donors": "C,H",
        "format": "cif",
        "input": "2HYV.cif",  # or None if using pdb
        "metal": None,
        "overwright": True,
        "pdb": None,          # or "2HYV" if using pdb
        "threshold": 2.8,
        "workdir": "./findgeo",
        "excluded-metals": Mg,Ca",
        "java-exe": "/path/to/java/executable",
        "findgeo-jar": "/path/to/FindGeo.jar"
    }
    rFG = RunFindGeo(d_args)
    rFG.run()
    """
    def __init__(self, d_args):
        self.d_args = d_args
        if not self.validateArgs():
            logger.error("invalid arguments")
            sys.exit(1)

    def validateArgs(self):
        """
        validate arguments in d_args
        required keys in d_args:
        excluded-donors, format, input, metal, overwright, pdb, threshold, workdir, excluded-metals, java-exe, findgeo-jar
        1. java-exe and findgeo-jar must exist as files
        2. format must be either 'cif' or 'pdb'
        3. if metal is specified, it must be a valid chemical symbol (1 or 2 letters)
        4. threshold must be a float between 1.0 and 4.0
        5. workdir must be a valid directory, create it if it does not exist
        6. either input or pdb must be specified, but not both
        7. if input is specified, it must exist as a file
        8. if pdb is specified, it must be a valid PDB code (4 alphanumeric characters)
        Return True if all validations pass, otherwise False

        :return: True if all validations pass, otherwise False
        :rtype: bool
        """
        if not os.path.exists(self.d_args['java-exe']):
            logger.error("java executable not found: %s", self.d_args['java-exe'])
            return False
        if not os.path.exists(self.d_args['findgeo-jar']):
            logger.error("FindGeo jar file not found: %s", self.d_args['findgeo-jar'])
            return False
        if self.d_args['format'] not in ['cif', 'pdb']:
            logger.error("invalid format: %s", self.d_args['format'])
            return False
        if self.d_args['metal'].lower() != 'all':
            if self.d_args['metal'] and len(self.d_args['metal']) > 2:
                logger.error("invalid metal symbol: %s", self.d_args['metal'])
                return False
        if self.d_args['excluded-metals'] != 'None':
            l_metal = self.d_args['excluded-metals'].split(',')
            for metal in l_metal:
                if len(metal) > 2:
                    logger.error("invalid excluded-metals symbol: %s", self.d_args['excluded-metals'])
                    return False
        if self.d_args['threshold'] <= 1.0 or self.d_args['threshold'] >= 4.0:
            logger.error("invalid threshold: %s", self.d_args['threshold'])
            return False
        try:
            os.makedirs(self.d_args['workdir'], exist_ok=True)
        except Exception as e:
            logger.error("cannot create workdir: %s with error %s", self.d_args['workdir'], e)
            return False

        # validate input and pdb arguments and pick the non-empty one to use as input
        self.input = []
        if self.d_args['input'] and os.path.exists(self.d_args['input']):
            logger.info("using input file: %s", self.d_args['input'])
            self.input = ["--input", self.d_args['input']]
        else:
            logger.info("input file not found: %s, try pdb id input", self.d_args['input'])
            if self.d_args['pdb']:
                if len(self.d_args['pdb']) not in [4, 12]:
                    logger.error("invalid pdb id: %s", self.d_args['pdb'])
                    return False
                logger.info("using pdb id: %s", self.d_args['pdb'])
                self.input = ["--pdb", self.d_args['pdb'].lower()]
            else:
                logger.error("must specify either input file or pdb id")
                return False

        return True

    def run(self):
        """
        run FindGeo with arguments in d_args and self.input as input file or pdb id
        example command with local input file:
            /usr/local/opt/openjdk/bin/java
            -jar FindGeo.jar
            --input 2HYV.cif
            --excluded-donors C,H
            --format cif
            --threshold 2.8
            --workdir findgeo
            --excluded-metals Mg,Ca
            --overwrite
        example command with pdb id:
            /usr/local/opt/openjdk/bin/java
            -jar FindGeo.jar
            --pdb 2HYV
            --excluded-donors C,H
            --format cif
            --threshold 2.8
            --workdir findgeo
            --excluded-metals Mg,Ca
            --overwrite

        :return: stdout from FindGeo if successful, otherwise None
        :rtype: str or None
        """
        l_command = [self.d_args["java-exe"], "-jar", self.d_args["findgeo-jar"]]
        l_command.extend(self.input)  # get input from either --input or --pdb
        for arg in ['format', 'threshold', 'workdir']:
            if self.d_args[arg]:
                l_command.extend([f'--{arg}', str(self.d_args[arg])])
        if self.d_args["metal"] and self.d_args["metal"].lower() != 'all':
            l_command.extend(['--metal', self.d_args["metal"]])
        if self.d_args['overwright']:
            l_command.append('--overwrite')
        if self.d_args['excluded-donors'] != "C,H":
            l_command.extend(['--excluded-donors', self.d_args["excluded-donors"]])
        if self.d_args['excluded-metals'] != "None":
            l_command.extend(['--excluded-metals', self.d_args["excluded-metals"]])

        logger.info("to run FindGeo full command:\n %s", ' '.join(l_command))
        try:
            cmd_stdout = run_command(l_command)
            logger.info("finished running FindGeo command on %s", self.input)
            return cmd_stdout
        except MetalCommandExecutionError as e:
            logger.error(f"MetalCommandExecutionError: {e}")
            return None



# pylint: disable=logging-format-interpolation
import argparse
import logging
import os
import shutil
import subprocess
import tempfile
import time
from enum import Enum
from textwrap import dedent

from wwpdb.utils.config.ConfigInfo import ConfigInfo, getSiteId

logging.basicConfig(level=logging.DEBUG, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s")
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


def remove_file(file_path):
    if os.path.exists(file_path):
        try:
            os.remove(file_path)
        except Exception as _e:  # noqa: F841
            pass


class JobStatus(Enum):
    OOM = "OUT_OF_MEMORY"
    RUNNING = "RUNNING"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    CANCELLED = "CANCELLED"
    OTHER = "OTHER"


class RunRemote:
    def __init__(
        self,
        command,
        job_name,
        log_dir,
        run_dir=None,
        timeout=90,
        memory_limit=16000,
        number_of_processors=1,
        add_site_config=False,
        add_site_config_database=False,
    ):
        self.command = command
        self.job_name = job_name
        self.log_dir = log_dir
        self.run_dir = run_dir
        self.timeout = 90 if timeout == 0 else str(timeout)
        self.memory_limit = str(memory_limit)
        self.number_of_processors = str(number_of_processors)
        self.add_site_config = add_site_config
        self.add_site_config_database = add_site_config_database

        if not self.run_dir:
            self.run_dir = tempfile.mkdtemp(prefix="run_remote_")  # this won't work as cluster nodes have different temp dirs
        self._shell_script = os.path.join(self.run_dir, "run_{}.sh".format(self.job_name))

        self.siteId = getSiteId()
        self.cI = ConfigInfo(self.siteId)
        self.pdbe_cluster_queue = str(self.cI.get("PDBE_CLUSTER_QUEUE"))
        self._stdout_file = os.path.join(self.log_dir, self.job_name + ".out")
        self._stderr_file = os.path.join(self.log_dir, self.job_name + ".err")

    def get_job_status_by_id(self, job_id) -> JobStatus:
        """Get the status of a single job by ID."""
        cmd = [
            "squeue",
            "--noheader",
            "-t",
            "all",
            "--Format",
            "State",
            "--jobs",
            str(job_id),
        ]
        squeue_output = subprocess.run(cmd, check=True, capture_output=True)
        status_text = squeue_output.stdout.decode("utf-8").strip()

        if status_text in ["FAILED", "TIMEOUT"]:
            return JobStatus.FAILED
        if status_text == "OUT_OF_MEMORY":
            return JobStatus.OOM
        if status_text == "COMPLETED":
            return JobStatus.COMPLETED
        if status_text in ["RUNNING", "PENDING", "CONFIGURING", "COMPLETING"]:
            return JobStatus.RUNNING
        if status_text == "CANCELLED":
            return JobStatus.CANCELLED
        return JobStatus.OTHER

    def requeue_job(self, job_id):
        """Requeue a single job."""
        cmd = ["scontrol", "requeue", str(job_id)]
        subprocess.run(cmd, check=True)
        logger.info(f"Requeued failed job {job_id}")

    def monitor(self, job_id, frequency=10):
        """Monitor a job by ID, requeueing if it fails."""
        logging.info(f"Monitoring job {job_id}")  # noqa: LOG015

        while True:
            status = self.get_job_status_by_id(job_id)

            if status in (JobStatus.CANCELLED, JobStatus.FAILED, JobStatus.OOM):
                logger.warning(f"Job {job_id} failed with status {status}")
                break
            if status == JobStatus.COMPLETED:
                logger.info(f"Job {job_id} completed successfully")
                break
            else:
                logger.debug(f"Job {job_id} status: {status}")

            time.sleep(frequency)

        return self.get_job_status_by_id(job_id)

    def _build_sbatch_command(self, command):
        sbatch_args = [
            "sbatch",
            "--job-name=%s" % self.job_name,
            "--partition=%s" % self.pdbe_cluster_queue,
            "--cpus-per-task=%s" % self.number_of_processors,
            "--mem=%s" % self.memory_limit,
            "--time=%s" % self.timeout,
            "--chdir=%s" % self.run_dir,
            "--output=%s" % self._stdout_file,
            "--error=%s" % self._stderr_file,
        ]

        with open(self._shell_script, "w") as f:
            cmd = f"""\
            #!/bin/bash
            set -e
            export XDG_RUNTIME_DIR={self.run_dir}
            {command}
            """
            f.write(dedent(cmd))
            f.flush()
        os.chmod(self._shell_script, 0o775)

        sbatch_args += [self._shell_script]
        return sbatch_args

    def _cleanup(self):
        if self.run_dir.startswith("/tmp/run_remote_"):  # noqa: S108
            shutil.rmtree(self.run_dir)

    def _source_site_config(self, database=False):
        suffix = ""
        if database:
            suffix = "--database"

        site_config_path = self.cI.get("TOP_WWPDB_SITE_CONFIG_DIR")
        site_loc = self.cI.get("WWPDB_SITE_LOC")
        site_config_command = ". {}/init/env.sh --siteid {} --location {} {} > /dev/null".format(site_config_path, self.siteId, site_loc, suffix)

        return "{}; {}".format(site_config_command, self.command)

    def run(self, retries=3):
        status = JobStatus.OTHER
        wf_command = self.command

        if self.add_site_config_database or self.add_site_config:
            wf_command = self._source_site_config(database=self.add_site_config_database)

        while retries > 0:
            sbatch_cmd = self._build_sbatch_command(command=wf_command)
            logger.info(" ".join(sbatch_cmd))

            output = subprocess.run(sbatch_cmd, check=True, capture_output=True)
            job_id = int(output.stdout.decode("utf-8").split()[-1])
            logger.debug(f"Submitted: {job_id}")

            self._cleanup()
            status = self.monitor(job_id=job_id)

            if status == JobStatus.COMPLETED:
                break

            if status == JobStatus.OOM:
                self.memory_limit = str(int(self.memory_limit) * 2)

            logger.info(f"Retrying job {job_id} with memory limit {self.memory_limit}")
            retries -= 1

        return status


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest="comm")

    parser_run = subparsers.add_parser("run")
    parser_run.add_argument("-d", "--debug", help="debugging", action="store_const", dest="loglevel", const=logging.DEBUG, default=logging.INFO)
    parser_run.add_argument("--command", help="command to run", type=str, required=True)
    parser_run.add_argument("--job_name", help="name for the job", type=str, required=True)
    parser_run.add_argument("--log_dir", help="directory to store log file in", type=str, required=True)
    parser_run.add_argument("--run_dir", help="directory to run", type=str)
    parser_run.add_argument("--memory_limit", help="starting memory limit", type=int, default=16000)
    parser_run.add_argument("--num_processors", help="number of processors", type=int, default=1)
    parser_run.add_argument("--add_site_config", help="add site config to command", action="store_true")
    parser_run.add_argument("--add_site_config_with_database", help="add site config with database to command", action="store_true")

    args = parser.parse_args()

    logger.info(f"Running command: {args.comm}")
    if args.comm == "run":
        run_remote = RunRemote(
            command=args.command,
            job_name=args.job_name,
            log_dir=args.log_dir,
            run_dir=args.run_dir,
            memory_limit=args.memory_limit,
            number_of_processors=args.num_processors,
            add_site_config=args.add_site_config,
            add_site_config_database=args.add_site_config_with_database,
        )
        status_ret = run_remote.run()
        logger.info(f"Job finished with status: {status_ret}")



import argparse
import logging
import os
import shutil
import sys
import tempfile

import gemmi

from wwpdb.utils.dp.electron_density.common_functions import convert_mdb_to_binary_cif, run_command_and_check_output_file

logger = logging.getLogger(__name__)


class XrayVolumeServerMap:
    def __init__(
        self,
        coord_path,
        binary_map_out,
        node_path,
        volume_server_pack_path,
        volume_server_query_path,
        working_dir,
        two_fofc_mmcif_map_coeff_in,
        fofc_mmcif_map_coeff_in,
    ):
        self.coord_path = coord_path
        self.binary_map_out = binary_map_out
        self.node_path = node_path
        self.volume_server_pack_path = volume_server_pack_path
        self.volume_server_query_path = volume_server_query_path
        self.working_dir = working_dir
        self.two_fofc_mmcif_map_coeff_in = two_fofc_mmcif_map_coeff_in
        self.fofc_mmcif_map_coeff_in = fofc_mmcif_map_coeff_in

        # intermediate files
        self.mdb_map_path = os.path.join(self.working_dir, "mdb_map.mdb")
        self.two_fo_fc_map = os.path.join(self.working_dir, "2fofc.map")
        self.fo_fc_map = os.path.join(self.working_dir, "fofc.map")

    def run_process(self):
        ok = False
        ok1 = self.gemmi_sf2map(
            sf_mmcif_in=self.two_fofc_mmcif_map_coeff_in,
            map_out=self.two_fo_fc_map,
            f_column="pdbx_FWT",
            phi_column="pdbx_PHWT",
        )
        ok2 = self.gemmi_sf2map(
            sf_mmcif_in=self.fofc_mmcif_map_coeff_in,
            map_out=self.fo_fc_map,
            f_column="pdbx_DELFWT",
            phi_column="pdbx_DELPHWT",
        )
        if ok1 and ok2:
            ok = self.make_maps_to_serve_with_volume_server(
                two_fofc_map_in=self.two_fo_fc_map,
                fofc_map_in=self.fo_fc_map,
            )
            if ok:
                ok = self.convert_mdb_map_to_binary_cif()
            else:
                logger.error("making mdb maps failed")
        else:
            logger.error("making maps failed")

        return ok

    def gemmi_sf2map(self, sf_mmcif_in, map_out, f_column, phi_column):
        """
        converts input mmCIF file map coefficients to map
        :param sf_mmcif_in: mmCIF structure factor input file
        :param map_out: map output file
        :param f_column: F column
        :param phi_column: PHI column
        :return: True if worked, False if failed
        """
        st = gemmi.read_structure(self.coord_path, format=gemmi.CoorFormat.Mmcif)
        fbox = st.calculate_fractional_box(margin=5)
        if sf_mmcif_in:
            if os.path.exists(sf_mmcif_in):
                doc = gemmi.cif.read(sf_mmcif_in)  # pylint: disable=no-member
                rblocks = gemmi.as_refln_blocks(doc)
                if f_column in rblocks[0].column_labels() and phi_column in rblocks[0].column_labels():  # pylint: disable=unsubscriptable-object
                    ccp4 = gemmi.Ccp4Map()
                    ccp4.grid = rblocks[0].transform_f_phi_to_map(f_column, phi_column)  # pylint: disable=unsubscriptable-object
                    ccp4.update_ccp4_header(2, True)
                    ccp4.set_extent(fbox)
                    ccp4.write_ccp4_map(map_out)

                    if os.path.exists(map_out):
                        return True
                    logger.error("output map file {} missing".format(map_out))  # pylint: disable=logging-format-interpolation)
                else:
                    logger.error("{} {} columns not found in mmCIF {}".format(f_column, phi_column, sf_mmcif_in))  # pylint: disable=logging-format-interpolation
            else:
                logger.error("cannot find input file {}".format(sf_mmcif_in))  # pylint: disable=logging-format-interpolation)

                # command = "{} sf2map -f {} -p {} {} {}".format(gemmi_path, f_column, phi_column, mmcif_in, map_out)
                # return self.run_command(command=command, output_file=map_out)
        logger.error("converting {} to {} failed".format(sf_mmcif_in, map_out))  # pylint: disable=logging-format-interpolation)
        return False

    def make_maps_to_serve_with_volume_server(
        self,
        two_fofc_map_in,
        fofc_map_in,
    ):
        if not self.node_path:
            logger.error("node path not set")
            return False
        if self.volume_server_pack_path:
            return self.make_volume_server_map(
                two_fofc_map_in=two_fofc_map_in,
                fofc_map_in=fofc_map_in,
            )
        return False

    def make_volume_server_map(self, two_fofc_map_in, fofc_map_in):
        """
        make map for Volume server to serve
        :param: map_in: input map file
        :param: map_out: output map file
        :return: True if worked, False if failed
        """
        if not self.node_path:
            logger.error("node path not set")
            return False
        if not self.volume_server_pack_path:
            logger.error("volume server executable not set")
            return False
        if not os.path.exists(self.volume_server_pack_path):
            logger.error("volume server executable not found at {}".format(self.volume_server_pack_path))  # pylint: disable=logging-format-interpolation
            return False
        if os.path.exists(two_fofc_map_in) and os.path.exists(fofc_map_in):
            command = "{} {} xray {} {} {}".format(self.node_path, self.volume_server_pack_path, two_fofc_map_in, fofc_map_in, self.mdb_map_path)
            return run_command_and_check_output_file(command=command, workdir=None, process_name="make mdb_map", output_file=self.mdb_map_path)
        return False

    def convert_mdb_map_to_binary_cif(self):
        return convert_mdb_to_binary_cif(
            map_id="x_ray_volume",
            source_id="x-ray",
            output_file=self.binary_map_out,
            working_dir=self.working_dir,
            mdb_map_path=self.mdb_map_path,
            volume_server_query_path=self.volume_server_query_path,
            node_path=self.node_path,
            detail=4,
        )


def run_process_with_gemmi(
    node_path,
    coord_file,
    two_fofc_mmcif_map_coeff_in,
    fofc_mmcif_map_coeff_in,
    binary_map_out,
    volume_server_pack_path=None,
    volume_server_query_path=None,
):
    """
    Process 2fo-fc and fo-fc mmCIF files and convert to maps for volume server
    :param node_path: path to node executable
    :param coord_file: path to mmCIF coordinate file
    :param two_fofc_mmcif_map_coeff_in: input 2Fo-Fc map coefficient mmCIF file
    :param fofc_mmcif_map_coeff_in: input Fo-Fc map coefficient mmCIF file
    :return: True if worked, False if failed
    """

    if not volume_server_pack_path:
        logger.error("volume-server-pack path must be set")
        return False

    if not volume_server_query_path:
        logger.error("volume-server-query path must be set")
        return False

    if not coord_file:
        logger.error("coordinate file must be provided")
        return False

    if not node_path:
        logger.error("node not set")
        return False

    if not os.path.exists(node_path):
        logger.error("node not found: {}".format(node_path))  # pylint: disable=logging-format-interpolation
        return False

    if not os.path.exists(two_fofc_mmcif_map_coeff_in) or not os.path.exists(fofc_mmcif_map_coeff_in):
        logger.error("input mmcif files not found: {} or {}".format(two_fofc_mmcif_map_coeff_in, fofc_mmcif_map_coeff_in))  # pylint: disable=logging-format-interpolation)
        return False

    run_working_directory = tempfile.mkdtemp()
    logger.debug("working directory: {}".format(run_working_directory))  # pylint: disable=logging-format-interpolation
    xrsm = XrayVolumeServerMap(
        coord_path=coord_file,
        node_path=node_path,
        volume_server_pack_path=volume_server_pack_path,
        binary_map_out=binary_map_out,
        working_dir=run_working_directory,
        two_fofc_mmcif_map_coeff_in=two_fofc_mmcif_map_coeff_in,
        fofc_mmcif_map_coeff_in=fofc_mmcif_map_coeff_in,
        volume_server_query_path=volume_server_query_path,
    )

    ret = xrsm.run_process()

    # Cleanup
    shutil.rmtree(run_working_directory, ignore_errors=True)

    return ret


def main():  # pragma: no cover
    FORMAT = "[%(levelname)s]-%(module)s.%(funcName)s: %(message)s"
    logging.basicConfig(format=FORMAT)

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--two_fofc_mmcif_map_coeff_in",
        help="2fofc mmCIF containing map coefficients",
        type=str,
        required=True,
    )
    parser.add_argument(
        "--fofc_mmcif_map_coeff_in",
        help="fofc mmCIF containing map coefficients",
        type=str,
        required=True,
    )
    parser.add_argument(
        "--binary_map_out",
        help="binary map output file name",
        type=str,
        required=True,
    )
    parser.add_argument("--node_path", help="node program path", type=str, required=True)
    parser.add_argument("--coordinate_file", help="mmCIF coordinate file", type=str, required=True)
    parser.add_argument("--volume_server_pack_path", help="volume-server-pack path", type=str, required=True)
    parser.add_argument("--volume_server_query_path", help="volume-server-query path", type=str, required=True)
    parser.add_argument("--keep_working", help="Keep working directory", action="store_true")
    parser.add_argument(
        "-d",
        "--debug",
        help="debugging",
        action="store_const",
        dest="loglevel",
        const=logging.DEBUG,
        default=logging.INFO,
    )

    args = parser.parse_args()
    logger_in = logging.getLogger()
    logger_in.setLevel(args.loglevel)

    ok = run_process_with_gemmi(
        node_path=args.node_path,
        volume_server_pack_path=args.volume_server_pack_path,
        volume_server_query_path=args.volume_server_query_path,
        two_fofc_mmcif_map_coeff_in=args.two_fofc_mmcif_map_coeff_in,
        fofc_mmcif_map_coeff_in=args.fofc_mmcif_map_coeff_in,
        coord_file=args.coordinate_file,
        binary_map_out=args.binary_map_out,
    )

    if not ok:
        sys.exit(1)


if __name__ == "__main__":  # pragma: no cover
    main()



import json
import logging
import os
import shlex
import shutil
import subprocess

logger = logging.getLogger(__name__)


def run_command(command, process_name, workdir=None):
    """
    run a command and check the output
    :param str command: the command to be run
    :param str process_name: a name for the process
    :param str None workdir: a path or None for a working directory
    :return bool: True if exit status is zero, False otherwise
    """
    try:
        logger.debug(command)
        command_list = shlex.split(command)
        if workdir and workdir is not None:
            process = subprocess.Popen(command_list, cwd=workdir)
        else:
            process = subprocess.Popen(command_list)
        out, err = process.communicate()
        if out:
            logger.info(out)
        if err:
            logger.error(err)
        rc = process.returncode
        if rc != 0:
            logger.error("process failed: {}".format(process_name))  # pylint: disable=logging-format-interpolation
            return False
        logger.info("process worked: {}".format(process_name))  # pylint: disable=logging-format-interpolation
        return True
    except Exception as e:  # noqa: BLE001
        logger.error(e)
    return False


def run_command_and_check_output_file(command, process_name, output_file, workdir=None):
    """
    run a command and check the output file exists
    :param str command: the command to be run
    :param str None workdir: path or None
    :param str process_name: a name for the process
    :param str output_file: the output file to check for
    :return bool: True if the command exits with status 0 and the output file exists, False otherwise
    """
    if command and output_file:
        ret = run_command(command=command, workdir=workdir, process_name=process_name)
        if ret:
            logger.debug("checking for {}".format(output_file))  # pylint: disable=logging-format-interpolation
            if os.path.exists(output_file):
                logger.debug("file exists")
                return True
            logger.error("output file missing: {}".format(output_file))  # pylint: disable=logging-format-interpolation
        else:
            logger.error("command returned non-zero exit status")
    else:
        logger.error("either command or output file not set")

    return False


def convert_mdb_to_binary_cif(node_path, volume_server_query_path, map_id, source_id, mdb_map_path, output_file, working_dir, detail=4):
    query_kind = "cell"
    map_file_name = "{}_{}-{}_d{}.bcif".format(map_id, source_id, query_kind, detail)
    if not working_dir:
        working_dir = os.getcwd()
    temp_out_file = os.path.join(working_dir, map_file_name)
    json_filename = "conversion.json"
    json_content = [
        {
            "source": {"filename": mdb_map_path, "name": map_id, "id": source_id},
            "query": {"kind": query_kind},
            "params": {"detail": detail, "asBinary": True},
            "outputFolder": working_dir,
            "outputFilename": map_file_name,
        }
    ]
    if working_dir:
        working_json = os.path.join(working_dir, json_filename)
    else:
        working_json = json_filename
    with open(working_json, "w") as out_file:
        json.dump(json_content, out_file)
    command = "{} {} --jobs {}".format(node_path, volume_server_query_path, working_json)
    ret = run_command_and_check_output_file(command=command, process_name="mdb_to_binary_cif", workdir=working_dir, output_file=temp_out_file)
    if ret:
        output_folder = os.path.dirname(output_file)
        if output_folder:
            if not os.path.exists(output_folder):
                os.makedirs(output_folder)
        shutil.copy(temp_out_file, output_file)
        logger.debug("output file {}".format(output_file))  # pylint: disable=logging-format-interpolation
        return True
    return False



import argparse
import logging
import os
import shutil
import sys
import tempfile

from wwpdb.utils.config.ConfigInfo import getSiteId

from wwpdb.utils.dp.RcsbDpUtility import RcsbDpUtility

logger = logging.getLogger()


class DensityWrapper:
    def __init__(self, site_id=None):
        self.__site_id = site_id or getSiteId()

    def convert_xray_density_map(self, coord_file, in_2fofc_cif, in_fofc_cif, out_binary_volume, working_dir):
        logging.info("Converting X-ray maps to binary cif")  # noqa: LOG015
        logging.debug(working_dir)  # noqa: LOG015
        rdb = RcsbDpUtility(tmpPath=working_dir, siteId=self.__site_id, verbose=True)
        rdb.imp(coord_file)
        rdb.addInput(name="two_fofc_cif", value=in_2fofc_cif, type="file")
        rdb.addInput(name="one_fofc_cif", value=in_fofc_cif, type="file")
        rdb.op("xray-density-bcif")
        rdb.exp(out_binary_volume)
        rdb.cleanup()

        if out_binary_volume:
            if os.path.exists(out_binary_volume):
                return True
        return False

    def convert_em_volume(self, in_em_volume, out_binary_volume, working_dir):
        logging.info("Converting EM maps to binary cif")  # noqa: LOG015
        logging.debug(working_dir)  # noqa: LOG015
        rdb = RcsbDpUtility(tmpPath=working_dir, siteId=self.__site_id, verbose=True)
        rdb.imp(in_em_volume)
        rdb.op("em-density-bcif")
        rdb.exp(out_binary_volume)
        rdb.cleanup()

        if out_binary_volume:
            if os.path.exists(out_binary_volume):
                return True
        return False


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--em_map", help="EM map", type=str)
    parser.add_argument("--binary_map_out", help="Output filename of binary map", type=str, required=True)
    parser.add_argument("--debug", help="debugging", action="store_const", dest="loglevel", const=logging.DEBUG, default=logging.INFO)

    args = parser.parse_args()
    # logger = logging.getLogger()
    logger.setLevel(args.loglevel)

    if not sys.argv[1:]:
        parser.print_help()
        sys.exit()

    dw = DensityWrapper()
    working_dir = tempfile.mkdtemp()
    if args.em_map:
        dw.convert_em_volume(in_em_volume=args.em_map, out_binary_volume=args.binary_map_out, working_dir=working_dir)

    shutil.rmtree(working_dir)


if __name__ == "__main__":
    main()



# Author:  Chenghua Shao
# Date:    2025-11-10
# Updates:

"""
This module orchestrates the execution of Acedrg, MetalCoord update mode, and Servalcat to
process the input ligand CCD file metal based on provided marcromolecular structure file,
output a ligand CIF file with updated ideal coordinates and charges,
tegether with a json report summarizing the metal coordination.
"""

import argparse
import logging
import os
import sys
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wwpdb.utils.dp.metal.metalcoord.runAcedrg import RunAcedrg  # noqa: E402
    from wwpdb.utils.dp.metal.metalcoord.runMetalCoord import RunMetalCoord  # noqa: E402
    from wwpdb.utils.dp.metal.metalcoord.runServalcat import RunServalcat  # noqa: E402
    from wwpdb.utils.dp.metal.metalcoord.parseMetalCoord import ParseMetalCoord  # noqa: E402
else:
    sys.path.append(os.path.dirname(os.path.abspath(__file__)))
    from runAcedrg import RunAcedrg  # noqa: E402
    from runMetalCoord import RunMetalCoord  # noqa: E402
    from runServalcat import RunServalcat  # noqa: E402
    from parseMetalCoord import ParseMetalCoord  # noqa: E402

logger = logging.getLogger(__name__)
# logging.basicConfig(format="%(asctime)s - %(levelname)s - %(message)s")
# logger.setLevel(logging.DEBUG)


def callAcedrg(d_args_acedrg):
    """
    Call Acedrg with the provided arguments and return the output CIF file path.
    :param d_args_acedrg: Dictionary of arguments for running Acedrg.
    :type d_args_acedrg: dict
    :returns: Path to the generated CIF file if successful, otherwise None.
    :rtype: str or None
    """

    try:
        rAG = RunAcedrg(d_args_acedrg)
    except Exception as e:
        logger.error(e)
        return None

    cmd_stdout = rAG.run()
    logger.info(cmd_stdout)

    fp_acedrg_cif = os.path.join(d_args_acedrg["out"] + ".cif")
    if os.path.exists(fp_acedrg_cif):
        return fp_acedrg_cif
    else:
        return None


def callMetalCoord(d_args_metalcoord):
    """
    Run MetalCoord in "update" mode and return output file paths.
    Initializes a RunMetalCoord instance with the provided argument dictionary,
    runs MetalCoord in "update" mode, and if the expected CIF output is not
    produced, retries with a fallback option (clearing the 'pdb' input in the
    RunMetalCoord arguments). The function then checks for the presence of the
    files "metalcoord.cif" and "metalcoord.cif.json" in the supplied work directory
    and returns their paths when both are present.
    :param d_args_metalcoord: Dictionary of arguments for RunMetalCoord. Must
        include at least the "workdir" key indicating where output files are
        written; other keys are passed to RunMetalCoord.
    :type d_args_metalcoord: dict
    :returns: A tuple (fp_metalcoord_cif, fp_metalcoord_json) with full paths to
        the CIF and JSON output files if both exist; (None, None) if the expected
        files are not both present; or None if initializing RunMetalCoord raises
        an exception (the exception is logged).
    :rtype: tuple(str, str) or (None, None) or None
    :side-effects: Invokes external MetalCoord runs, writes output files to the
        given workdir, and logs command output and any errors.
    """

    try:
        rMC = RunMetalCoord(d_args_metalcoord)
    except Exception as e:
        logger.error(e)
        return None
    rMC.setInputMode("update")
    cmd_stdout1 = rMC.run()  # 1st MetalCoord run based on PDB model
    logger.info(cmd_stdout1)

    fp_metalcoord_cif = os.path.join(d_args_metalcoord["workdir"], "metalcoord.cif")
    if not os.path.exists(fp_metalcoord_cif):
        rMC.d_args["pdb"] = None
        cmd_stdout2 = rMC.run()  # 2nd MetalCoord run by the option 'most_common'
        logger.info(cmd_stdout2)

    fp_metalcoord_json = os.path.join(d_args_metalcoord["workdir"], "metalcoord.cif.json")

    if os.path.exists(fp_metalcoord_cif) and os.path.exists(fp_metalcoord_json):
        return (fp_metalcoord_cif, fp_metalcoord_json)
    else:
        return (None, None)


def callServalcat(d_args_servalcat):
    """
    Call Servalcat to process and update a CIF file.
    This function constructs and runs a RunServalcat instance using the provided
    argument dictionary. It logs the command output and then checks for an output
    CIF file based on the ``output_prefix`` entry in ``d_args_servalcat``.
    :param d_args_servalcat: Dictionary of arguments forwarded to RunServalcat.
        Must include the key ``output_prefix`` whose value is used to form the
        expected output filename "<output_prefix>_updated.cif".
    :type d_args_servalcat: dict
    :returns: Path to the updated CIF file if it exists after running Servalcat,
        otherwise ``None``.
    :rtype: str or None
    :raises: Exceptions raised during RunServalcat construction are logged and
        suppressed; the function returns ``None`` in that case.
    :notes: The function logs errors and info via the module logger and does not
        raise exceptions for run failures.
    """

    try:
        rST = RunServalcat(d_args_servalcat)
    except Exception as e:
        logger.error(e)
        return None
    cmd_stdout = rST.run()
    logger.info(cmd_stdout)
    fp_servalcat_cif = d_args_servalcat["output_prefix"] + "_updated.cif"

    if os.path.exists(fp_servalcat_cif):
        return fp_servalcat_cif
    else:
        return None


def main():
    """
    run Acedrg-MetalCoord-Servalcat, then parse the output and generate a report json file in stats mode.
    Example usages:
    > python runMetalCoordUpdate.py --input 0KA.cif --pdb 4DHV.cif
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("-a", "--acedrg_exe", help="Acedrg executable file", type=str, default=None)
    parser.add_argument("-b", "--metalcoord_exe", help="MetalCoord executable file", type=str, default=None)
    parser.add_argument("-c", "--servalcat_exe", help="Servalcat executable file", type=str, default=None)
    parser.add_argument("-w", "--workdir", help="Directory to write outputs. Default is metalcoord subfolder in the current folder", type=str, default="metalcoord")
    parser.add_argument("-i", "--input", help="Ligand cif file", type=str, required=True)
    parser.add_argument("-p", "--pdb", help="PDB code or pdb file", type=str, default=None)
    parser.add_argument("-t", "--threshold", help="Procrustes distance threshold.", type=float, default=0.3)
    args = parser.parse_args()

    # run Acedrg
    d_args_acedrg = {}
    d_args_acedrg["acedrg_exe"] = args.acedrg_exe
    d_args_acedrg["mmcif"] = args.input
    d_args_acedrg["out"] = os.path.join(args.workdir, "acedrg")
    fp_acedrg_cif = callAcedrg(d_args_acedrg)
    if not fp_acedrg_cif:
        logger.error("Acedrg failed, STOP without output")
        sys.exit(1)

    # run MetalCoord
    d_args_metalcoord = {}
    d_args_metalcoord["metalcoord_exe"] = args.metalcoord_exe
    d_args_metalcoord["workdir"] = args.workdir
    d_args_metalcoord["input"] = fp_acedrg_cif  # use Acedrg output as input
    d_args_metalcoord["pdb"] = args.pdb
    d_args_metalcoord["threshold"] = args.threshold
    (fp_metalcoord_cif, fp_metalcoord_json) = callMetalCoord(d_args_metalcoord)
    if not fp_metalcoord_cif:
        logger.error("MetalCoord update mode failed, STOP without output")
        sys.exit(1)

    # run Servalcat
    d_args_servalcat = {}
    d_args_servalcat["servalcat_exe"] = None
    d_args_servalcat["update_dictionary"] = fp_metalcoord_cif  # use MetalCoord output as input
    d_args_servalcat["output_prefix"] = os.path.join(args.workdir, "servalcat")
    fp_servalcat_cif = callServalcat(d_args_servalcat)
    if not fp_servalcat_cif:
        logger.error("Servalcat failed, STOP without output")
        sys.exit(1)

    if not fp_metalcoord_json:
        logger.error("No MetalCoord output json")
        sys.exit(1)

    logger.info("to parse MetalCoord results from %s", fp_metalcoord_json)
    pMC = ParseMetalCoord()
    if pMC.read(fp_metalcoord_json):
        pMC.parse()
        output_json = os.path.join(args.workdir, "metalcoord_report.json")
        pMC.report(output_json)
        logger.info("MetalCoord results written to %s", output_json)
    else:
        logger.error("failed to read MetalCoord results at %s, no output", fp_metalcoord_json)


if __name__ == "__main__":
    main()



# Author:  Chenghua Shao
# Date:    2025-11-10
# Updates:

"""
Utility functions to run metal commands with logging and error handling.
This module provides a function to execute shell commands, log their output in a separate log file
from the main application log, and handle errors by raising custom exceptions.
The command-specific log can be used for debugging the 3rd party metal tools such as MetalCoord and FindGeo.
"""

import subprocess
import logging
from datetime import datetime
import os

# logger = logging.getLogger(__name__)


class MetalCommandExecutionError(Exception):
    """Raised when a metal command execution fails, e.g. FindGeo and MetalCoord failure."""
    def __init__(self, cmd, code=None, stderr=None, stdout=None):
        self.cmd = cmd
        self.code = code
        self.stderr = stderr
        self.stdout = stdout
        message = f"Command {cmd} failed with exit code {code}"
        if stderr:
            message += f"\nStderr:\n{stderr.strip()}"
        super().__init__(message)


def setup_logger(name="command_runner", log_dir="metal_command_logs"):
    """Use this only when an existing logger is not used for run_command() function below
    Create or retrieve a configured logger.
    """
    os.makedirs(log_dir, exist_ok=True)
    logger = logging.getLogger(name)

    if not logger.handlers:  # prevent duplicate handlers if called multiple times
        logger.setLevel(logging.DEBUG)
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        log_path = os.path.join(log_dir, f"cmd_{timestamp}.log")

        # File handler
        fh = logging.FileHandler(log_path, encoding="utf-8")
        fh.setLevel(logging.DEBUG)

        # Console handler, use for debugging
        ch = logging.StreamHandler()
        ch.setLevel(logging.WARNING)

        # Format
        fmt = logging.Formatter(
            fmt="%(asctime)s [%(levelname)s] %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )
        fh.setFormatter(fmt)
        ch.setFormatter(fmt)

        logger.addHandler(fh)
        logger.addHandler(ch)

        logger.info(f"Logging to file: {log_path}")

    return logger


def run_command(cmd, logger=None):
    """Run a local command and raise CommandExecutionError on failure."""
    if logger is None:
        logger = setup_logger()

    logger.info(f"▶ Running command: {' '.join(cmd)}")

    try:
        result = subprocess.run(
            cmd,
            check=True,
            capture_output=True,
            text=True
        )
        logger.debug(f"STDOUT:\n{result.stdout.strip()}")
        logger.info("✅ Command completed successfully.")
        return result.stdout

    except FileNotFoundError as e:
        logger.error(f"❌ Binary not found: {e}")
        raise MetalCommandExecutionError(cmd, None, stderr=str(e)) from e

    except subprocess.CalledProcessError as e:
        logger.error(f"❌ Command failed (exit code {e.returncode})")
        if e.stdout:
            logger.debug(f"STDOUT:\n{e.stdout.strip()}")
        if e.stderr:
            logger.error(f"STDERR:\n{e.stderr.strip()}")
        raise MetalCommandExecutionError(e.cmd, e.returncode, e.stderr, e.stdout) from e

    except Exception as e:
        logger.exception("❌ Unexpected error during command execution")
        raise MetalCommandExecutionError(cmd, None, stderr=str(e)) from e


# def main():
#     logger = setup_logger(log_dir="log_test")
#     try:
#         output = run_command(["ls", "/nonexistent"], logger)
#     except MetalCommandExecutionError as e:
#         logger.error(f"MetalCommandExecutionError, Handled error: {e}")


# if __name__ == "__main__":
#     main()



##
# File:    PdbxChemShiftReport.py
# Author:  jdw
# Date:    11-Sep-2014
# Version: 0.001
#
# Updated:
#
##
"""Read diagnostics in the chemical shift diagnostics report."""

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2,9"
__version__ = "V0.01"

import logging
import sys

# from mmcif.api.PdbxContainers import *
from mmcif.io.PdbxReader import PdbxReader

logger = logging.getLogger(__name__)


class PdbxChemShiftReport:
    def __init__(self, inputPath, verbose=False, log=sys.stderr):  # noqa: ARG002 pylint: disable=unused-argument
        self.__myContainerList = []
        self.__read(inputPath)

    def __read(self, inputPath):
        """Read status file"""
        try:
            self.__myContainerList = []
            with open(inputPath) as ifh:
                pRd = PdbxReader(ifh)
                pRd.read(self.__myContainerList)
            return True
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return False

    def getStatus(self):
        return self.__get("pdbx_shift_check", "status")

    def getWarnings(self):
        return self.__get("pdbx_shift_check_warning_message", "text")

    def getErrors(self):
        return self.__get("pdbx_shift_check_error_message", "text")

    def __get(self, categoryName, attributeName):
        retVal = []
        try:
            c0 = self.__myContainerList[0]
            catObj = c0.getObj(categoryName)
            if catObj is None:
                return retVal
            _nm, aList, rList = catObj.get()

            if attributeName in aList:
                idx = aList.index(attributeName)
                for r in rList:
                    retVal.append(str(r[idx]).strip())
        except Exception:  # noqa: BLE001
            pass

        return retVal


if __name__ == "__main__":
    pass



"""Functionailty to calculate the complexit of coordinate model"""

import argparse
import logging
import os
import sys

from mmcif.api.DataCategory import DataCategory
from mmcif.api.PdbxContainers import DataContainer
from mmcif.io.IoAdapterCore import IoAdapterCore as IoAdapter

logger = logging.getLogger(__name__)


class PdbxModelCompletity:
    def __init__(self, threshold=1e6):
        self.__data = {}
        self.__threshold = threshold

    def calculate(self, fpath):
        """Calculates complexity of coordinate file in fpath.  Returns True on success.  Returns False if file does not exist"""

        if not os.path.exists(fpath):
            logger.error("Filename %s does not exist", fpath)
            return False

        io = IoAdapter()
        cL = io.readFile(fpath, selectList=["entity", "pdbx_struct_assembly"])

        if not cL or len(cL) < 1:
            logger.error("Error processing complexity %s", fpath)
            return False

        b0 = cL[0]

        # Determine oligo count - use first row of pdbx_struct_assembly if present
        oligo_count = 1
        if b0.exists("pdbx_struct_assembly"):
            cObj = b0.getObj("pdbx_struct_assembly")

            if "oligomeric_count" in cObj.getAttributeList():
                val = cObj.getValueOrDefault("oligomeric_count", rowIndex=0, defaultValue="1")
                try:
                    valI = int(val)
                    oligo_count = valI
                except ValueError:
                    pass

        # Calculate complexity
        entry_complexity = 0
        non_poly_complexity = 0
        polymer_complexity = 0

        if b0.exists("entity"):
            cObj = b0.getObj("entity")

            # Ensure real data
            for attr in ["type", "pdbx_number_of_molecules", "formula_weight"]:
                if attr not in cObj.getAttributeList():
                    logger.error("Entity category missing attribute %s", attr)
                    return False

            for row in range(cObj.getRowCount()):
                etype = cObj.getValueOrDefault("type", row, "unknown")
                num_mol = cObj.getValueOrDefault("pdbx_number_of_molecules", row, "1")
                fw = cObj.getValueOrDefault("formula_weight", row, "unknown")

                try:
                    num_mol_int = int(num_mol)
                    fw_val = float(fw)
                except ValueError:
                    logger.error("Error converting values %s %s", num_mol, fw)

                if etype == "polymer":
                    entity_complexity = num_mol_int * fw_val * oligo_count
                    entry_complexity += entity_complexity
                    polymer_complexity += entity_complexity
                else:
                    # water, ligands, branched chain carbo, etc
                    entity_complexity = num_mol_int * fw_val
                    entry_complexity += entity_complexity
                    non_poly_complexity += entity_complexity

            is_complex = True if entry_complexity > self.__threshold else False  # noqa: SIM210

            self.__data = {
                "is_complex": is_complex,
                "entry_complexity": entry_complexity,
                "polymer_complexity": polymer_complexity,
                "non_poly_complexity": non_poly_complexity,
            }

        return True

    def write_output(self, fpath):
        """Writes out the"""

        c0 = DataContainer("complexity")

        is_complex = str(self.__data.get("is_complex", False))
        entry_complexity = str(self.__data.get("entry_complexity", 0))
        polymer_complexity = str(self.__data.get("polymer_complexity", 0))
        non_poly_complexity = str(self.__data.get("non_poly_complexity", 0))
        threshold = f"{self.__threshold:.2e}"

        attrlist = [
            "is_complex",
            "entry_complexity",
            "polymer_complexity",
            "non_poly_complexity",
            "complex_threshold",
        ]
        data = [
            [
                is_complex,
                entry_complexity,
                polymer_complexity,
                non_poly_complexity,
                threshold,
            ]
        ]

        cat = DataCategory("pdbx_complexity", attrlist, data)
        c0.append(cat)

        clist = [c0]

        io = IoAdapter()
        io.writeFile(fpath, clist)


def main():
    logging.basicConfig(level=logging.INFO)
    parser = argparse.ArgumentParser(
        prog="PdbxModelComplexity.py",
        description="Calculates the complexity of a model file and outputs a CIF file with information",
    )
    parser.add_argument("--model", required=True, help="model file to calculate complexity")
    parser.add_argument("--output", required=True, help="output file to record complexity")
    parser.add_argument(
        "--threshold",
        type=float,
        default=1e8,
        help="threshold value to use as cutoff (1.0E+6 default)",
    )
    args = parser.parse_args()

    threshold = args.threshold
    modelpath = args.model
    output = args.output

    pmc = PdbxModelCompletity(threshold=threshold)
    ok = pmc.calculate(modelpath)
    if ok:
        pmc.write_output(output)
    else:
        print("Could not parse input file %s" % modelpath)  # noqa: T201
        sys.exit(1)

    sys.exit(0)


if __name__ == "__main__":
    main()



# Author:  Chenghua Shao
# Date:    2025-11-10
# Updates:

"""
This module provides utility functions to read reference data related to metal ions,
including oxidation states, coordination numbers, and coordinate class mappings from CSV files.
"""

import csv
import os

REF_PATH = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "metal_ref")


def readRefRedOx():
    """
    The function expects a file named ``metal_oxidation_state.csv`` located in the directory specified by ``REF_PATH``.
    The CSV file should be tab-delimited and contain at least the columns ``Metals``, ``Redox active``, and ``Oxidation state``.
    :returns: A tuple containing two dictionaries:
    :rtype: tuple
    :raises FileNotFoundError: If the CSV file does not exist.
    :raises KeyError: If expected columns are missing in the CSV file.
    """

    filepath = os.path.join(REF_PATH, "metal_oxidation_state.csv")
    d_redox = {}
    d_oxi = {}
    with open(filepath) as f:
        reader = csv.DictReader(f, delimiter="\t")
        for d_row in reader:
            metal = d_row['Metals'].strip()
            d_redox[metal] = d_row['Redox active'].strip()
            d_oxi[metal] = d_row["Oxidation state"]
    return (d_redox, d_oxi)


def readRefCoordNum():
    """
    Reads the metal coordination number reference data from a CSV file and returns a dictionary mapping metal names to their coordination numbers.
    The CSV file is expected to be located at REF_PATH/metal_coordination_number.csv, with tab-delimited columns 'Metals' and 'Coordination numbers'.
    :returns: Dictionary mapping metal names (str) to lists of coordination numbers (str).
    :rtype: dict
    """

    filepath = os.path.join(REF_PATH, "metal_coordination_number.csv")
    d_coord_num = {}
    with open(filepath) as f:
        reader = csv.DictReader(f, delimiter="\t")
        for d_row in reader:
            metal = d_row['Metals'].strip()
            coord_num = d_row['Coordination numbers'].strip()
            d_coord_num[metal] = coord_num.split(",")
    return d_coord_num


def readRefCoordMap(program):
    """
    Reads a CSV file containing coordinate class mappings and returns a dictionary mapping geometry names to their abbreviations and PDB geometry names for a specified program.

    :param str program: The name of the program to select the appropriate columns from the CSV file.
    :returns: A dictionary where keys are geometry names (lowercase) and values are dictionaries with 'abbr' (abbreviation, uppercase) and 'pdb_geom' (PDB geometry name, lowercase).
    :rtype: dict

    .. note::
        - Skips rows where the geometry name is 'na'.
        - Prints a message if a duplicate geometry name is encountered.
        - Assumes the CSV file is located at ``REF_PATH/coord_classes_mapping_abbr.csv`` and contains columns named ``Name {program}``, ``Abbreviation {program}``, and ``Name PDB``.
    """

    filepath = os.path.join(REF_PATH, "coord_classes_mapping_abbr.csv")
    d_coord_map = {}
    geom_header = f"Name {program}"
    abbr_header = f"Abbreviation {program}"
    pdb_header = "Name PDB"
    with open(filepath) as f:
        reader = csv.DictReader(f, delimiter=",")
        for d_row in reader:
            geom = d_row[geom_header].strip().lower()
            if geom == "na":
                continue
            abbr = d_row[abbr_header].strip().upper()
            pdb_geom = d_row[pdb_header].strip().lower()
            if geom in d_coord_map:
                print("duplicate geometry %s", geom)
                continue
            d_coord_map[geom] = {}
            d_coord_map[geom]['abbr'] = abbr
            d_coord_map[geom]['pdb_geom'] = pdb_geom
    return d_coord_map


# def main():
#     d_coord_num = readRefCoordNum()
#     print(d_coord_num)
#     d_coord_map_fg = readRefCoordMap("FindGeo")
#     print(d_coord_map_fg)
#     d_coord_map_mc = readRefCoordMap("metalCoord")
#     print(d_coord_map_mc)
#     (d_redox, d_oxi) = readRefRedOx()
#     print(d_redox)
#     print(d_oxi)

# if __name__ == "__main__":
#     main()



import sys
import traceback

from wwpdb.utils.dp.RcsbDpUtility import RcsbDpUtility


class DepositorSyncUtil:
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        self.__verbose = verbose
        self.__lfh = log
        self.__reqObj = reqObj
        self.__sObj = None
        self.__sessionId = None
        self.__sessionPath = None
        self.__siteId = str(self.__reqObj.getValue("WWPDB_SITE_ID"))
        self.__getSession()

    def syncWithDatabase(self, depId, modelFilePath):
        self.__lfh.write("+DepositorSyncUtil.syncWithDatabase() - syncing depositor data in %s with database for %s\n" % (modelFilePath, depId))

        try:
            dp = RcsbDpUtility(tmpPath=self.__sessionPath, siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
            dp.addInput(name="depId", value=depId)
            dp.addInput(name="modelFilePath", value=modelFilePath)
            dp.op("sync-depositors")
            return
        except:  # noqa: E722 pylint: disable=bare-except
            self.__lfh.write("DepositorSyncUtil::syncWithDatabase(): failing, with exception.\n")
            traceback.print_exc(file=self.__lfh)

    def __getSession(self):
        """Join existing session or create new session as required."""
        self.__sObj = self.__reqObj.newSessionObj()
        self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        if self.__verbose:
            self.__lfh.write("------------------------------------------------------\n")
            self.__lfh.write("+DepositorSyncUtil.__getSession() - creating/joining session %s\n" % self.__sessionId)
            self.__lfh.write("+DepositorSyncUtil.__getSession() - session path %s\n" % self.__sessionPath)



import argparse
import logging
import os
import sys

import gemmi
from mmcif.api.DataCategory import DataCategory
from mmcif.io.IoAdapterCore import IoAdapterCore
from wwpdb.io.locator.PathInfo import PathInfo
from wwpdb.utils.config.ConfigInfo import getSiteId

logger = logging.getLogger()


def get_model_file(depid, version_id, mileStone=None, siteId=None):
    if siteId is None:
        siteId = getSiteId()
    pi = PathInfo(siteId, sessionPath=".", verbose=True, log=sys.stderr)
    mmcif = pi.getModelPdbxFilePath(dataSetId=depid, fileSource="archive", versionId=version_id, mileStone=mileStone)
    logging.debug("mmcif file path: %s", mmcif)  # noqa: LOG015
    return mmcif


def get_center_of_mass(data_block):
    try:
        st = gemmi.make_structure_from_block(data_block)
        model = st[0]
        center_of_mass = model.calculate_center_of_mass()
        return center_of_mass
    except Exception as e:  # noqa: BLE001
        logging.error(e)  # noqa: LOG015
        return False


def get_deposition_ids(file):
    deposition_ids = []
    with open(file) as f:
        for line in f:
            deposition_ids.append(line.strip())
    return deposition_ids


def process_entry(file_in, file_out):
    try:
        cif_file = gemmi.cif.read(file_in)  # pylint: disable=no-member
        data_block = cif_file[0]
    except Exception as e:  # noqa: BLE001
        logger.error("Failed to read cif file in Gemmi")
        logger.error(e)
        return 1

    logging.info("Finding Centre of Mass")  # noqa: LOG015
    com = get_center_of_mass(data_block)
    if not com:
        return 1

    try:
        io = IoAdapterCore()
        ccL = io.readFile(file_in)
    except Exception as e:  # noqa: BLE001
        logger.error("Failed to read cif file using IoAdapterCore %s", e)
        return 1

    if len(ccL) == 0:
        logger.error("No data parsed from file")
        return 1

    # First block only
    b0 = ccL[0]

    obj = b0.getObj("struct")
    # If category does not exist
    if obj is None:
        # Need entry.id
        eid = "XXXX"
        eobj = b0.getObj("entry")
        if eobj:
            if "id" in eobj.getAttributeList():
                eid = eobj.getValue("id", 0)
        obj = DataCategory("struct")
        obj.appendAttribute("entry_id")
        obj.setValue(eid, "entry_id", 0)
        ccL[0].append(obj)

    newdata = [["pdbx_center_of_mass_x", com.x], ["pdbx_center_of_mass_y", com.y], ["pdbx_center_of_mass_z", com.z]]
    for [it, val] in newdata:
        if it not in obj.getAttributeList():
            obj.appendAttribute(it)
        obj.setValue(str(val), it, 0)

    try:
        logging.info("Writing mmcif file: %s", file_out)  # noqa: LOG015
        ret = io.writeFile(file_out, ccL)
        if not ret:
            logger.info("Writing failed error %s", ret)
            return 1
    except Exception as e:  # noqa: BLE001
        logger.error("Failed to write ccif file in IoAdapater %s", e)
        return 1

    # existing_data = data_block.get_mmcif_category('_struct.')
    # new_data = {
    #     **existing_data,
    #     'pdbx_center_of_mass_x': [com.x],
    #     'pdbx_center_of_mass_y': [com.y],
    #     'pdbx_center_of_mass_z': [com.z]
    # }
    # logging.info("Writing mmcif file: %s", file_out)
    # try:
    #     data_block.set_mmcif_category('_struct.', new_data)
    #     cif_file.write_file(file_out)
    # except Exception as e:
    #     logger.error("Failed to write cif file in Gemmi")
    #     logger.error(e)
    #     return 1
    return 0


def calculate_for_list(args, siteId=None):
    logging.info("Calculating for list of entries")  # noqa: LOG015
    deposition_ids = get_deposition_ids(args.list)
    failed_dep_ids = []
    for depid in deposition_ids:
        logging.info("Calculating for Dep ID: %s ", depid)  # noqa: LOG015
        latest_model = get_model_file(depid, "latest", siteId=siteId)
        next_model = get_model_file(depid, "next", siteId=siteId)
        result = process_entry(latest_model, next_model)
        if result:
            logger.info("Failed to Calculate Centre of Mass for %s", depid)
            failed_dep_ids.append(depid)
    return failed_dep_ids


def calculate_for_file(args):
    logging.info("Calculating for a single file")  # noqa: LOG015
    result = process_entry(args.model_file_in, args.model_file_out)
    if result:
        logger.info("Failed to Calculate Centre of Mass")


def main(args):
    if args.list and os.path.isfile(args.list):
        failures = calculate_for_list(args)
        if len(failures) > 0:
            logger.info("Failed Dep Ids: \n%s", "\n".join(failures))
            return 1

    elif os.path.isfile(args.model_file_in):
        calculate_for_file(args)
        return 0
    else:
        logger.error("Input model file doesn't exists")
        return 1

    return 0


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--model-file-in", help="Coordinate file to add centre of Mass", type=str)
    parser.add_argument("-o", "--model-file-out", help="Output Coordinate file, with added items", type=str)
    parser.add_argument("-l", "--list", help="list of Deposition Ids to calculate and append centre of Mass", type=str)
    parser.add_argument("-log", "--log-level", help="Log level", type=str, default="INFO")
    parser.add_argument("-s", "--siteid", help="optional siteId", type=str)

    args = parser.parse_args()
    logger.setLevel(args.log_level)

    if not sys.argv[1:]:
        parser.print_help()
        sys.exit()

    main(args)


if "__main__" in __name__:
    parse_args()



# Author:  Chenghua Shao
# Date:    2025-11-10
# Updates:

"""
Wrapper to parse MetalCoord output json file
"""

import json
import os
import sys
import logging
from collections import OrderedDict
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wwpdb.utils.dp.metal.metal_util.readRef import readRefCoordNum, readRefCoordMap, readRefRedOx
else:
    sys.path.append(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "metal_util"))
    from readRef import readRefCoordNum, readRefCoordMap, readRefRedOx  # noqa: E402

logger = logging.getLogger(__name__)


class ParseMetalCoord:
    """Wrapper to parse MetalCoord output files
    """
    def __init__(self):
        self.d_coord_num = readRefCoordNum()
        self.d_coord_map = readRefCoordMap("metalCoord")
        (self.d_redox, self.d_oxi) = readRefRedOx()
        self.data = None
        self.l_sites = []

    def read(self, fp_metalcoord):
        """load json data from the MetalCoord output json file
        :param fp_metalcoord: MetalCoord output json file
        :return: bool True for successful loading or non-empty data
        """
        try:
            with open(fp_metalcoord, "r") as f:
                self.data = json.load(f)
                logger.info("JSON loaded successfully from %s", fp_metalcoord)
                if self.data:
                    logger.info("json file %s is not empty", fp_metalcoord)
                    return True
                else:
                    logger.warning("json file %s is empty, STOP process", fp_metalcoord)
                    return False
        except FileNotFoundError:
            logger.error("Error: File not found at %s", fp_metalcoord)
            return False

        except PermissionError:
            logger.error("Error: Permission denied when trying to open %s.", fp_metalcoord)
            return False

        except json.JSONDecodeError as e:
            logger.error("Error: Failed to decode JSON for %s — %s", fp_metalcoord, e)
            return False

        except Exception as e:
            logger.error("Unexpected error in loading data from %s: %s", fp_metalcoord, e)
            return False

    def parse(self):
        """
        parse MetalCoord output folder to extract top hit coordination geometry for each site.
        1. iterate through each site folder in self.folder
        2. for each site folder, parse MetalCoord.out to get top hit coordination geometry
        3. parse MetalCoord.input to get metal atom information
        4. store the results in self.l_sites
        5. sort self.l_sites by metal, chain, residue, sequence, icode
        """
        self.filter()
        logger.info("finished filtering the MetalCoord output json")
        if self.l_sites:
            self.amend()
            logger.info("finished adding info to the output json")
            self.sort()
        else:
            logger.warning("no metal sites parsed")

    def filter(self):
        """
        Extract a compact, top-hit summary for each metal site from self.data.
        """
        for d_site in self.data:
            d_site_filtered = {}
            for key1 in ["metal", "metalElement", "chain", "residue", "sequence", "icode", "altloc"]:
                d_site_filtered[key1] = d_site[key1]

            # find the best coordination with lowest procruste value
            threshold = 1.0
            d_tophit = {}
            for d_coord in d_site["ligands"]:
                score = d_coord["procrustes"]
                if score < threshold:
                    d_tophit = d_coord
                    threshold = score

            # once best coordination is found, move class assignment one level up
            for key2 in ["class", "descriptor", "procrustes", "coordination", "count"]:
                d_site_filtered[key2] = d_tophit[key2]
            d_site_filtered["class_abbr"] = d_tophit["class_abr"]

            # record only coordination ligands in the new "sphere" record
            d_site_filtered["sphere"] = d_tophit["order"]

            self.l_sites.append(d_site_filtered)

    def amend(self):
        """
        amend d_tophit with additional information from reference data
        1. add generic geometry name from the coordination class mapping reference
        2. check whether the coordination number is allowd in the coordination number reference
        3. add RedOx active marker

        :param d_tophit: dict with top hit information
        :return: amended dict with additional information
        """
        for d_tophit in self.l_sites:
            geom = d_tophit["class"]
            if geom in self.d_coord_map:
                pdb_geom = self.d_coord_map[geom]["pdb_geom"]
                d_tophit["class_generic"] = pdb_geom
            else:
                d_tophit["class_generic"] = ""

            metal = d_tophit["metalElement"]
            if metal in self.d_coord_num:
                allowed_coord_num = self.d_coord_num.get(metal)
                if str(d_tophit["coordination"]) in allowed_coord_num:
                    d_tophit["coordination_number_allowed"] = "YES"
                else:
                    d_tophit["coordination_number_allowed"] = "NO"
            else:
                d_tophit["coordination_number_allowed"] = ""

            if metal in self.d_redox:
                d_tophit["redox_active"] = self.d_redox.get(metal)
            else:
                d_tophit["redox_active"] = ""

            if metal in self.d_oxi:
                d_tophit["oxidation_state"] = self.d_oxi.get(metal)
            else:
                d_tophit["oxidation_state"] = ""

    def sort(self):
        """
        sort self.l_sites
        """
        key_order = ["metal", "metalElement", "chain", "residue", "sequence", "icode", "altloc",
                     "coordination", "class", "class_abbr", "class_generic", "descriptor", "procrustes", "count",
                     "coordination_number_allowed", "redox_active", "oxidation_state", "sphere"]
        l_sorted = []
        for d_row in self.l_sites:
            d_row_sorted = OrderedDict((key, d_row[key]) for key in key_order if key in d_row)
            l_sorted.append(d_row_sorted)
        self.l_sites = l_sorted

    def report(self, filepath_json):
        """
        write self.l_sites to a json file

        :param filepath_json: path to output json file
        """
        logger.info("to write report to %s", filepath_json)
        with open(filepath_json, "w") as file:
            json.dump(self.l_sites, file, indent=4)



# Author:  Chenghua Shao
# Date:    2025-11-10
# Update:

"""
Run FindGeo and parse results.
Summary:
1. Run FindGeo with user provided arguments.
2. Parse FindGeo output files.
3. Generate a report json file summarizing the results.
"""

import argparse
import logging
import os
import sys
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wwpdb.utils.dp.metal.findgeo.runFindGeo import RunFindGeo  # noqa: E402
    from wwpdb.utils.dp.metal.findgeo.parseFindGeo import ParseFindGeo  # noqa: E402
else:
    sys.path.append(os.path.dirname(os.path.abspath(__file__)))
    from runFindGeo import RunFindGeo  # noqa: E402
    from parseFindGeo import ParseFindGeo  # noqa: E402

logger = logging.getLogger(__name__)
# logging.basicConfig(format="%(asctime)s - %(levelname)s - %(message)s")
# logger.setLevel(logging.DEBUG)


def main():
    """
    run FindGeo and take arguments exactly like the command line for findgeo,
    then parse the output and generate a report json file.
    Example usages:
    > python runFindGeo.py --java-exe /path/to/java --findgeo-jar /path/to/FindGeo.jar --input 2HYV.cif
    > python runFindGeo.py --java-exe /path/to/java --findgeo-jar /path/to/FindGeo.jar --pdb 2HYV
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("-e", "--excluded-donors", help="Chemical symbols of the atoms (separated by commas) excluded from metal ligands. Default is 'C,H' ", type=str, default="C,H")
    parser.add_argument("-f", "--format", help="Local file format (i.e. cif or pdb).", type=str, default="cif")
    parser.add_argument("-i", "--input", help="Local PDB/mmCIF local file.", type=str, default=None)
    parser.add_argument("-m", "--metal", help="Chemical symbol of the metal of interest. Default is all metals.", type=str, default="All")
    parser.add_argument("-o", "--overwright", help="Overwrite existing files and directories.", action="store_true", default=True)
    parser.add_argument("-p", "--pdb", help="Local input PDB file or PDB code of input PDB file to be downloaded from the web.", type=str, default=None)
    parser.add_argument("-t", "--threshold", help="Coordination distance threshold. Default is 2.8 A.", type=float, default=2.8)
    parser.add_argument("-w", "--workdir", help="Directory to write outputs. Default is findgeo subfolder in the current folder", type=str, default="findgeo")
    parser.add_argument("-x", "--excluded-metals", help="Metal symbols (separated by commas) excluded from the analysis.", type=str, default="None")
    parser.add_argument("-b", "--java-exe", help="Java executable filepath", type=str, required=True)
    parser.add_argument("-a", "--findgeo-jar", help="FindGeo compiled jar filepath", type=str, required=True)
    args = parser.parse_args()

    l_args = ["excluded-donors", "format", "input", "metal", "overwright", "pdb", "threshold", "workdir", "excluded-metals", "java-exe", "findgeo-jar"]
    d_args = {}
    for arg in l_args:
        key = arg.replace("-", "_")
        d_args[arg] = getattr(args, key)

    logger.info("run FindGeo with %s", d_args)
    rFG = RunFindGeo(d_args)
    cmd_stdout = rFG.run()

    if cmd_stdout:
        logger.info(cmd_stdout)
        logger.info("run FindGeo finished")
        logger.info("parse FindGeo results")
        pFG = ParseFindGeo(d_args["workdir"], input_format=d_args["format"])
        pFG.parse()
        output_json = os.path.join(d_args["workdir"], "findgeo_report.json")
        pFG.report(output_json)
        logger.info("FindGeo results written to %s", output_json)
    else:
        logger.error("run FindGeo failed, no output json")


if __name__ == "__main__":
    main()



# Author:  Chenghua Shao
# Date:    2025-11-10
# Updates:

"""
This script runs MetalCoord in stats mode for specified ligands and PDB files,
parses the output, and generates a report JSON file.
"""

import argparse
import logging
import os
import sys
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wwpdb.utils.dp.metal.metalcoord.runMetalCoord import RunMetalCoord  # noqa: E402
    from wwpdb.utils.dp.metal.metalcoord.parseMetalCoord import ParseMetalCoord  # noqa: E402
else:
    sys.path.append(os.path.dirname(os.path.abspath(__file__)))
    from runMetalCoord import RunMetalCoord  # noqa: E402
    from parseMetalCoord import ParseMetalCoord  # noqa: E402

logger = logging.getLogger(__name__)
# logging.basicConfig(format="%(asctime)s - %(levelname)s - %(message)s")
# logger.setLevel(logging.DEBUG)


def main():
    """
    run MetalCoord and take arguments exactly like the command line for MetalCoord,
    then parse the output and generate a report json file in stats mode.
    Example usages:
    > python runMetalCoord.py --ligands 0KA --pdb 4DHV.cif --max_size 100
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("-b", "--metalcoord_exe", help="MetalCoord executable file", type=str, default=None)
    parser.add_argument("-w", "--workdir", help="Directory to write outputs. Default is metalcoord subfolder in the current folder", type=str, default="metalcoord")
    parser.add_argument("-l", "--ligands", help="Ligand code or comma-separated multiple codes, e.g. 0KA,NCO", type=str, required=True)
    parser.add_argument("-p", "--pdb", help="PDB code or pdb file", type=str, required=True)
    parser.add_argument("-x", "--max_size", help="Maximum sample size for statistics.", type=int, default=100)
    parser.add_argument("-t", "--threshold", help="Procrustes distance threshold for finding COD reference.", type=float, default=0.3)
    args = parser.parse_args()

    l_args = ["metalcoord_exe", "workdir", "pdb", "max_size", "threshold"]

    l_ligand = args.ligands.split(",")  # split multiple ligands if applicable
    l_json_outputs = []

    # run MetalCoord for each ligand
    for ligand in l_ligand:
        d_args = {"ligand": ligand}
        for arg in l_args:
            d_args[arg] = getattr(args, arg)

        try:
            rMC = RunMetalCoord(d_args)
        except Exception as e:
            logger.error(e)
            sys.exit(1)
        rMC.setInputMode("stats")
        cmd_stdout = rMC.run()  # 1st MetalCoord run based on PDB model
        logger.info(cmd_stdout)

        fp_metalcoord_json = os.path.join(d_args["workdir"], d_args["ligand"] + ".json")
        if fp_metalcoord_json:
            l_json_outputs.append(fp_metalcoord_json)
        else:
            logger.error("MetalCoord stats mode failed, STOP without output")
            sys.exit(1)

    # parse MetalCoord results and generate report
    pMC = ParseMetalCoord()
    for fp_metalcoord_json in l_json_outputs:
        logger.info("to parse MetalCoord results from %s", fp_metalcoord_json)
        if pMC.read(fp_metalcoord_json):
            pMC.parse()
        else:
            logger.error("failed to read MetalCoord results at %s, no output", fp_metalcoord_json)
        output_json = os.path.join(d_args["workdir"], "metalcoord_report.json")
        pMC.report(output_json)
        logger.info("MetalCoord results written to %s", output_json)


if __name__ == "__main__":
    main()



# Author:  Chenghua Shao
# Date:    2025-11-10
# Updates:

"""
Wrapper to run MetalCoord with arguments similar to command line
Two modes are supported: stats and update
"""

import logging
import os
import sys
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wwpdb.utils.dp.metal.metal_util.run_command import MetalCommandExecutionError, run_command  # noqa: E402
else:
    sys.path.append(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "metal_util"))
    from run_command import MetalCommandExecutionError, run_command  # noqa: E402

logger = logging.getLogger(__name__)


class RunMetalCoord:
    """Wrapper to run MetalCoord with arguments similar to command line
    Example usage:
    d_args = {
        "metalcoord_exe": "/path/to/metalcoord/MetalCoord",
        "ligand": "1PT",
        "pdb": "1PG9.cif",
        "workdir": "metalcoord",
        "max_size": 100,
    }
    rMC = RunMetalCoord(d_args)
    rMC.run()
    """
    def __init__(self, d_args):
        self.d_args = d_args
        self.mode = None
        if not self.d_args["metalcoord_exe"]:
            logger.info("%s is called without explicit MetalCoord executable, to find in CCP4", self.__class__.__name__)
            ccp4_dir = os.getenv("CCP4", default=None)
            if not ccp4_dir:
                raise KeyError("Environment variable 'CCP4' not found")
            metalcoord_exe = os.path.join(ccp4_dir, "bin", "metalCoord")
            if not os.path.exists(metalcoord_exe):
                raise FileNotFoundError("MetalCoord executable not found in CCP4 bin/ folder")
            self.d_args["metalcoord_exe"] = metalcoord_exe
            logger.info("use MetalCoord executable at %s", metalcoord_exe)

    def setInputMode(self, mode):
        self.mode = mode  # stats or update

    def run(self):
        if self.mode == "stats":
            return self.runStats()
        if self.mode == "update":
            return self.runUpdate()

    def runStats(self):
        """
        run MetalCoord stats mode with arguments in d_args
        example command:
            MetalCoord stats
            --ligand 1PT
            --pdb 1PG9.cif  # accept both PDB ID and PDB/mmCIF file
            --output metalcoord/1PT.json
            --max_size 100
        :return: stdout from MetalCoord if successful, otherwise None
        :rtype: str or None
        """
        l_command = [self.d_args["metalcoord_exe"], "stats"]
        l_command.extend(["--ligand", self.d_args["ligand"].upper()])  # ensure ligand code is uppercase
        l_command.extend(["--pdb", self.d_args["pdb"]])
        l_command.extend(["--max_size", str(self.d_args["max_size"])])
        l_command.extend(["--threshold", str(self.d_args["threshold"])])
        fp_out = os.path.join(self.d_args["workdir"], f"{self.d_args['ligand']}.json")
        l_command.extend(["--output", fp_out])

        logger.info("to run MetalCoord full command:\n %s", ' '.join(l_command))
        try:
            cmd_stdout = run_command(l_command)
            logger.info("finished running MetalCoord stats mode on %s of %s", self.d_args["ligand"], self.d_args["pdb"])
            return cmd_stdout
        except MetalCommandExecutionError as e:
            logger.error(f"MetalCommandExecutionError: {e}")
            return None

    def runUpdate(self):
        """
        run MetalCoord stats mode with arguments in d_args, with two options to run:
        if a PDB model reference is provided, i.e. seld.d_args["pdb"] is not empty, run update based on model;
        if a PDB model reference is NOT provided, run update by most_common option.
        example commands:
        update based on a PDB model:
            MetalCoord update
            --input acedrg/1PT.cif
            --output metalcoord/1PT.cif
            --pdb 1PG9.cif
        update by the most_common option referring to geometry observed in the PDB archive
            MetalCoord update
            --input acedrg/1PT.cif
            --output metalcoord/1PT.cif
            --cif
            --cl most_common
        :return: stdout from MetalCoord if successful, otherwise None
        :rtype: str or None
        """
        l_command = [self.d_args["metalcoord_exe"], "update"]
        l_command.extend(["--input", self.d_args["input"]])
        fp_out = os.path.join(self.d_args["workdir"], "metalcoord.cif")
        l_command.extend(["--output", fp_out])
        l_command.extend(["--threshold", str(self.d_args["threshold"])])
        if self.d_args["pdb"]:
            logger.info("to run MetalCoord update mode based on model of %s", self.d_args["pdb"])
            l_command.extend(["--pdb", self.d_args["pdb"]])
        else:
            logger.info("to run MetalCoord update mode by most_common option without model")
            l_command.extend(["--cif", "--cl", "most_common"])

        logger.info("to run MetalCoord full command:\n %s", ' '.join(l_command))
        try:
            cmd_stdout = run_command(l_command)
            logger.info("finished running MetalCoord update mode on %s by %s", self.d_args["input"], self.d_args["pdb"])
            return cmd_stdout
        except MetalCommandExecutionError as e:
            logger.error(f"MetalCommandExecutionError: {e}")
            return None


def main():
    d_args = {"metalcoord_exe": None,
              "ligand": "0KA",
              "pdb": "4DHV",
              "workdir": "metalcoord",
              "max_size": 100,
              "threshold": 0.1
              }
    try:
        rMC = RunMetalCoord(d_args)
    except Exception as e:
        print(e)
        sys.exit(1)
    rMC.setInputMode("stats")
    cmd_stdout = rMC.run()
    print(cmd_stdout)


if __name__ == "__main__":
    main()
